@@ -1,4 +1,5 @@
 from __future__ import annotations
+import gzip
 
 import logging
 import os
@@ -10,6 +11,7 @@ from dataclasses import dataclass
 from datetime import datetime
 from inspect import signature
 from typing import TYPE_CHECKING, Any, Callable, Dict, List, Optional, Union, overload
+from ice.api.utils import caller_file
 
 import numpy as np
 from ice.core.graph import (ExecutableGraph, GraphOutputCache, InvalidURIError,
@@ -119,6 +121,7 @@ class Task(_Task):
         with set_grad_enabled(self.training):
             self.__call__impl(hypergraph, launcher)
         self.finished = True
+        self.egraph.timer_log_to_file()
 
     def __call__impl(self, hypergraph: "HyperGraph", launcher: ElasticLauncher):
         # maintain running progress.
@@ -128,6 +131,7 @@ class Task(_Task):
         self.launcher = launcher
         self.egraph: ExecutableGraph = hypergraph.select_egraph(self.tags)
         self.egraph.task = self
+        self.egraph.enable_timer("timer" in self.tags)
 
         if self.egraph is not hypergraph._last_executed_egraph:
             if hypergraph._last_executed_egraph is not None:
@@ -447,20 +451,22 @@ class HyperGraph:
         self._num_workers = 0
 
         self.entrypoint = None
+        self.extra_backup_paths = []
         self.grad_acc_steps = 1
-        self.init_autocast(autocast_enabled, grad_scaler)
+        self.init_autocast(autocast_enabled, autocast_dtype, grad_scaler)
 
-    def init_autocast(self, autocast_enabled=True, grad_scaler:Union[bool, GradScaler] = None):
+    def init_autocast(self, autocast_enabled=True, autocast_dtype=None, grad_scaler:Union[bool, GradScaler] = None):
         """Initialize autocast.
 
         Args:
             autocast_enabled (bool): If True, enable autocast.
+            autocast_dtype: ...
             grad_scaler (GradScaler): Gradient scaler.
 
         Raises:
             ValueError: If the autocast_dtype is not valid.
         """
-        self.autocast_kwds = dict(enabled=autocast_enabled, dtype=None)
+        self.autocast_kwds = dict(enabled=autocast_enabled, dtype=autocast_dtype)
         self.init_grad_scaler(grad_scaler if grad_scaler is not None else autocast_enabled)
     
     def is_autocast_enabled(self) -> bool:
@@ -481,6 +487,10 @@ class HyperGraph:
             ValueError: If the entrypoint is not valid.
         """
         self.entrypoint = entrypoint
+    
+    def backup_extra_path(self, relpath:str):
+        abspath = os.path.join(os.path.dirname(caller_file()), relpath)
+        self.extra_backup_paths.append(abspath)
 
     @overload
     def init_grad_scaler(self,
@@ -761,6 +771,14 @@ class HyperGraph:
         global_shared_events["debugger_start"] = launcher.events.debugger_start
         global_shared_events["debugger_end"] = launcher.events.debugger_end
         
+        # determine default autocast_dtype
+        if self.autocast_kwds["dtype"] is None:
+            import torch
+            if launcher.assigned_device.type == "cuda":
+                self.autocast_kwds["dtype"] = torch.float16
+            else:
+                self.autocast_kwds["dtype"] = torch.bfloat16
+
         # setup tensorboard
         from ice.llutil.board import BoardWriter
         if not in_main_process() and get_current_launcher().rank == 0:
@@ -805,7 +823,7 @@ class HyperGraph:
             if self.entrypoint is not None:
                 src_dir = os.path.join(run_id_dir, "src")
                 os.makedirs(src_dir, exist_ok=True)
-                _backup_source_files_to(self.entrypoint, src_dir)
+                _backup_source_files_to(self.entrypoint, self.extra_backup_paths, src_dir)
         
             self.run_info.run_id = run_id
             self.run_info.full_run_id = os.path.basename(run_id_dir)
@@ -893,6 +911,7 @@ class HyperGraph:
             import torch
             tqdm.write(f"Saving checkpoint to \"{save_to}\".")
             torch.save(_checkpoint, save_to)
+            return save_to
 
     def load_checkpoint(self, resume_from, strict=False, tags="*"):
         """Load the checkpoint.
