@@ -4,4 +4,8 @@ class IgnoreMe:
     
     def __call__(self, *args, **kwds): return self
     
-    def __getattr__(self, __name: str): return self
\ No newline at end of file
+    def __getattr__(self, __name: str): return self
+
+    def __bool__(self): return False
+
+    def __nonzero__(self): return self.__bool__()
\ No newline at end of file
